#!/usr/bin/env node
import { Separator, checkbox, input, select, confirm } from "@inquirer/prompts";
import { execSync, spawnSync } from "child_process";
import pkg from "fs-extra";
import chalk from "chalk";
import { dirname, join, resolve } from "path";
import which from "which";
import { checkConnectorBoilerplate, checkNodeVersion, copyDirectory, exitWithError, getCommand, isNetlifyCLIInstalled, isPackageManager, promptEditorHelper, subStep, } from "./utils.js";
import { example, GITIGNORE, createReadme, testEdgeFunction, BOILERPLATE_OPTIONS, testFunction, createDetails, } from "./constants.js";
import { generateUiBoilerplate, extensionUIDeps, extensionUIDevDeps, } from "./extension-ui-boilerplate/generate-ui-boilerplate.js";
import { uiMigration } from "./ui-migration.js";
import { fileURLToPath } from "url";
const { ensureDirSync, writeFileSync, existsSync } = pkg;
const isCLIInstalled = isNetlifyCLIInstalled();
if (!isCLIInstalled) {
    console.error("The Netlify CLI must be installed in order to build, test and deploy extensions. Please install it by running:");
    console.error("npm install -g netlify-cli");
    exitWithError("Netlify CLI is required");
}
if (process.argv.includes("--uiMigration")) {
    await uiMigration();
    process.exit(0);
}
checkNodeVersion();
try {
    const directory = await input({
        message: `${chalk.bold.cyan(`Step 1/6:`)} Where do you want to create the extension?`,
        default: ".",
    });
    const destination = resolve(process.cwd(), directory);
    if (existsSync(resolve(destination, "package.json"))) {
        exitWithError("There is seemingly already a package.json in this directory. Please remove it and try again.");
    }
    const packageName = await input({
        message: `${chalk.bold.cyan(`Step 2/6:`)} What is the slug of the extension? (e.g. my-extension)`,
        default: destination.split("/").pop(),
        validate: (input) => {
            // should only be lowercase letters, numbers, and dashes
            if (!/^[a-z0-9-]+$/.test(input)) {
                return "Package name can only contain lowercase letters, numbers, and dashes";
            }
            return true;
        },
    });
    const component = await select({
        message: `${chalk.bold.cyan(`Step 3/6:`)} Which boilerplate should be included?`,
        choices: [
            ...Object.values(BOILERPLATE_OPTIONS),
            "None",
            "[more than one]",
        ].map((choice) => ({
            name: choice,
            value: choice,
        })),
    });
    let boilerplate = [];
    while (!boilerplate.length) {
        boilerplate =
            component === `[more than one]`
                ? await checkbox({
                    message: `${chalk.bold.cyan(subStep)} Which boilerplate should be included?`,
                    choices: Object.values(BOILERPLATE_OPTIONS).map((choice) => ({
                        name: choice,
                        value: choice,
                    })),
                })
                : [component];
    }
    const hasEdgeFunctionsBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.edgeFunctions);
    const hasFunctionsBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.functions);
    const hasBuildBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.build);
    const hasConnectorBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.connector);
    const hasUIBoilerplate = boilerplate.includes(BOILERPLATE_OPTIONS.ui);
    const needsBuildEventHandlerSafeguard = hasBuildBoilerplate ||
        hasEdgeFunctionsBoilerplate ||
        hasFunctionsBoilerplate;
    const connectorSupports = await checkConnectorBoilerplate(hasConnectorBoilerplate);
    let addEdgeFunctionConfig = false;
    if (hasBuildBoilerplate && !hasEdgeFunctionsBoilerplate) {
        addEdgeFunctionConfig = await select({
            message: `${chalk.bold.cyan(subStep)} Are you planning to inject edge functions into the user's site?`,
            choices: [
                { name: "Yes", value: true },
                { name: "No", value: false },
            ],
        });
    }
    let confirmVsCodeConfigure = false;
    if (addEdgeFunctionConfig || hasEdgeFunctionsBoilerplate) {
        confirmVsCodeConfigure = await confirm({
            message: `${chalk.bold.cyan(subStep)} Would you like to configure VS Code to use Edge Functions?`,
            default: false,
        });
    }
    const packageManagers = (await Promise.all(["pnpm", "yarn", "npm"].map(async (binary) => {
        return {
            choice: binary,
            isInstalled: (await which(binary, { nothrow: true })) !== null,
        };
    }))).sort((a, b) => {
        const hasA = a.isInstalled;
        const hasB = b.isInstalled;
        return hasA === hasB ? 0 : hasA ? -1 : 1;
    });
    const packageManager = await select({
        message: `${chalk.bold.cyan(`Step 4/6:`)} Which package manager do you want to use?`,
        choices: packageManagers.map(({ choice, isInstalled }) => ({
            name: `${choice}${isInstalled ? "" : " (not installed)"}`,
            value: choice,
        })),
    });
    if (!isPackageManager(packageManager)) {
        exitWithError("Invalid package manager");
    }
    const permissions = new Set([
        {
            enabled: hasConnectorBoilerplate,
            permissions: [
                "site:read",
                "site:write",
                "user:read",
                "user:write",
                "env:read",
                "env:write",
            ],
        },
        {
            enabled: hasBuildBoilerplate,
            permissions: ["site:read", "site:write"],
        },
        {
            enabled: hasEdgeFunctionsBoilerplate,
            permissions: ["site:read", "site:write"],
        },
        {
            enabled: hasFunctionsBoilerplate,
            permissions: ["site:read", "site:write"],
        },
        {
            enabled: hasUIBoilerplate,
            permissions: ["site:read", "site:write", "ui:read", "ui:write"],
        },
    ]
        .filter(({ enabled }) => enabled)
        .flatMap(({ permissions }) => permissions));
    if (permissions.size === 0) {
        const checkedPermissions = await checkbox({
            message: "Select your permission scopes",
            choices: [
                new Separator(" = Site = "),
                { name: "read", value: "site:read" },
                { name: "write", value: "site:write" },
                new Separator(" = User = "),
                { name: "read", value: "user:read" },
                { name: "write", value: "user:write" },
                new Separator(" = Environment Variables = "),
                { name: "read", value: "env:read" },
                { name: "write", value: "env:write" },
            ],
        });
        for (const permission of checkedPermissions) {
            permissions.add(permission);
        }
    }
    const scripts = {
        build: "netlify-extension build --all",
        dev: "netlify-extension dev --open",
        "dev-connector": "netlify-extension dev --connector",
    };
    const extensionPackageJson = {
        name: packageName,
        version: "0.0.1",
        license: "UNLICENSED",
        main: "src/index.ts",
        type: "module",
        scripts,
        dependencies: {},
    };
    if (hasConnectorBoilerplate) {
        extensionPackageJson.scripts = {
            ...scripts,
            dev: hasUIBoilerplate || needsBuildEventHandlerSafeguard
                ? "netlify-extension dev --open"
                : "netlify-extension dev -c",
        };
    }
    const extensionTsConfig = {
        compilerOptions: {
            target: "ES2022" /* Set the JavaScript language version for emitted JavaScript and include compatible library declarations. */,
            module: "ES2022" /* Specify what module code is generated. */,
            moduleResolution: "bundler",
            strict: true,
            rootDir: ".",
        },
        exclude: ["node_modules", "dist"],
    };
    ensureDirSync(destination);
    if (confirmVsCodeConfigure) {
        await promptEditorHelper({ destination });
    }
    writeFileSync(resolve(destination, "package.json"), JSON.stringify(extensionPackageJson, null, 2));
    writeFileSync(resolve(destination, "tsconfig.json"), JSON.stringify(extensionTsConfig, null, 2));
    writeFileSync(resolve(destination, ".gitignore"), GITIGNORE.trim() + "\n");
    writeFileSync(resolve(destination, "README.md"), createReadme(component).trim() + "\n");
    writeFileSync(resolve(destination, "details.md"), createDetails(packageName, boilerplate.length > 0).trim() + "\n");
    const __dirname = dirname(fileURLToPath(import.meta.url));
    const srcDir = join(__dirname, "/extension-ui-boilerplate/templates/src/assets");
    const destDir = join(destination, "/assets/");
    copyDirectory(srcDir, destDir);
    console.log(`${chalk.bold.cyan(`Step 5/6:`)} Adding dependencies...`);
    const deps = ["@netlify/sdk@latest"];
    const devDeps = [
        "@netlify/netlify-plugin-netlify-extension@latest",
        "typescript@latest",
    ];
    if (hasUIBoilerplate ||
        needsBuildEventHandlerSafeguard ||
        hasConnectorBoilerplate) {
        deps.push(...extensionUIDeps);
        devDeps.push(...extensionUIDevDeps);
    }
    const addCommand = getCommand("add", packageManager).split(" ");
    const addDeps = spawnSync(addCommand[0], [addCommand[1], ...deps], {
        cwd: destination,
        stdio: "inherit",
    });
    if (addDeps.status !== 0) {
        exitWithError(`Failed to add dependencies. Please try again with ${packageManager}.`);
    }
    const addDevDeps = spawnSync(addCommand[0], [addCommand[1], ...devDeps, "-D"], {
        cwd: destination,
        stdio: "inherit",
    });
    if (addDevDeps.status !== 0) {
        exitWithError(`Failed to add dev dependencies. Please try again with ${packageManager}.`);
    }
    console.log(`${chalk.bold.cyan(`Step 6/6:`)} Initialising extension...`);
    const surfaceArgs = [];
    if (hasUIBoilerplate || needsBuildEventHandlerSafeguard) {
        surfaceArgs.push("--surface=extension-team-configuration");
        surfaceArgs.push("--surface=extension-top-level-site-configuration");
    }
    if (hasConnectorBoilerplate) {
        if (connectorSupports.visualEditor) {
            surfaceArgs.push("--surface=extension-visual-editor-configuration");
        }
        if (connectorSupports.connect) {
            surfaceArgs.push("--surface=extension-connect-configuration");
        }
    }
    execSync(getCommand("run", packageManager, [
        // use the local SDK bin if it exists
        process.env.LOCAL_SDK_BIN_PATH || "netlify-extension",
        "init",
        "-s",
        packageName,
        permissions.size > 0 ? `-p ${Array.from(permissions).join(",")}` : "",
        ...surfaceArgs,
    ]), {
        cwd: destination,
        env: {
            ...process.env,
        },
    });
    const BUILD_EVENT_HANDLER_SAFEGUARD_ENV_VAR = packageName.replace(/-/g, "_").toUpperCase() + "_ENABLED";
    ensureDirSync(resolve(destination, "src"));
    writeFileSync(resolve(destination, "src/index.ts"), "// Documentation: https://sdk.netlify.com/docs\n" +
        example({
            boilerplate,
            addEdgeFunctionConfig,
            connectorSupports,
            safeGuardEnvVar: BUILD_EVENT_HANDLER_SAFEGUARD_ENV_VAR,
        }));
    if (hasUIBoilerplate ||
        needsBuildEventHandlerSafeguard ||
        hasConnectorBoilerplate) {
        generateUiBoilerplate(destination, boilerplate, connectorSupports, BUILD_EVENT_HANDLER_SAFEGUARD_ENV_VAR);
    }
    if (addEdgeFunctionConfig ||
        hasEdgeFunctionsBoilerplate ||
        hasEdgeFunctionsBoilerplate ||
        hasFunctionsBoilerplate) {
        ensureDirSync(resolve(destination, "src/edge-functions"));
        writeFileSync(resolve(destination, "src/edge-functions/hello-world.ts"), testEdgeFunction);
    }
    if (hasFunctionsBoilerplate) {
        ensureDirSync(resolve(destination, "src/functions"));
        writeFileSync(resolve(destination, "src/functions/hello-world.mts"), testFunction);
    }
    if (destination) {
        ensureDirSync(destination);
        // check if the directory already has a git repository
        const gitStatus = spawnSync("git", ["status"], {
            cwd: destination,
        });
        if (gitStatus.status !== 0) {
            const gitInit = spawnSync("git", ["init"], {
                cwd: destination,
            });
            const gitAdd = spawnSync("git", ["add", "."], {
                cwd: destination,
            });
            const gitCommit = spawnSync("git", ["commit", "-m", "Initial commit from the Netlify SDK"], {
                cwd: destination,
            });
            const commands = [gitInit, gitAdd, gitCommit];
            if (commands.some((cmd) => cmd.error)) {
                exitWithError(`Error occurred while initializing git repository: ${gitInit.error || gitAdd.error || gitCommit.error}`);
            }
            if (commands.some((cmd) => cmd.status !== 0)) {
                exitWithError(`Git exited with code ${gitInit.status || gitAdd.status || gitCommit.status}. Please try again.`);
            }
            console.log("Initialised a git repository.");
        }
    }
    console.log(chalk.cyanBright("\nYour extension was created! Time to build! 🚀"));
    if (destination !== process.cwd()) {
        console.log("\nEnter your extension directory using", chalk.bold.cyan(`cd ${directory}`));
    }
    console.log("To start the development server, run", chalk.bold.cyan(`${packageManager} run dev`));
    console.log("Need help? Check out the documentation at", chalk.bold.cyan("https://sdk.netlify.com/docs"));
    console.log(`Or check the README.md file in your extension directory.`);
    console.log("\nDone! ✅");
}
catch (error) {
    if (error instanceof SyntaxError) {
        console.log(error.message);
    }
    else if (error instanceof ReferenceError) {
        console.log(error.message);
    }
    else if (error instanceof TypeError) {
        console.log(error.message);
    }
    else {
        console.log(chalk.bold.cyan("...gracefully exiting"));
        console.log(chalk.bold.cyan("See you later!"));
    }
}
//# sourceMappingURL=index.js.map